const height = 650;
const width = 650;
const margin = {
    left:70,
    top:50,
    bottom:50,
    right:300
}

//References: I used the following two:

//https://bl.ocks.org/heybignick/3faf257bbbbc7743bb72310d03b86ee8

//https://bl.ocks.org/mbostock/4062045

var edgeHash = {};
var matrix = []

const drag = simulation => {

    function dragstarted(event) {
        if (!event.active) simulation.alphaTarget(0.3).restart();
        event.subject.fx = event.subject.x;
        event.subject.fy = event.subject.y;
    }

    function dragged(event) {
        event.subject.fx = event.x;
        event.subject.fy = event.y;
    }

    function dragended(event) {
        if (!event.active) simulation.alphaTarget(0);
        event.subject.fx = null;
        event.subject.fy = null;
    }

    return d3.drag()
        .on("start", dragstarted)
        .on("drag", dragged)
        .on("end", dragended);
}


const loadData = async () => {
    try{
        const data = await d3.csv("food.csv")
        // reading from csv
        // processing data		
        const sources = data.map(d => d.new_id).filter((d,i,self) => i == self.indexOf(d))
        const targets = data.map(d => d.id).filter((d,i,self) => i == self.indexOf(d))
        const nodes = [...sources, ...targets].filter((d,i,self) => i == self.indexOf(d)).map(d => ({id:d, group:1}))
        const links = data.map(d => ({target:d.id, source:d.new_id, value: Math.ceil(Math.random(1) * 10), name:d.name}))


        // ADJACENCY MATRIX CODE      
        links.forEach(edge => {
            var id = edge.source + "-" + edge.target
            edgeHash[id] = edge
        })

        nodes.forEach((source, a) => {
            nodes.forEach((target, b) => {
                var grid = { id: source.id + "-" + target.id, x: b, y: a, weight: 0 };
                if (edgeHash[grid.id]) 
                    grid.weight = edgeHash[grid.id].value;
                matrix.push(grid)
            })
        })

        return { nodes, links }
    }
    catch(err){
        return err
    }
}

const render = data => {
    // svg container
    const svg = d3.select(".container")
        .attr("width", width)
        .attr("height", height)
        .style("background-color", "#f3f3f3")

    const simulation = d3.forceSimulation(data.nodes)
        .force("link", d3.forceLink(data.links).strength(0.01).id(d => d.id))
        .force("charge", d3.forceManyBody())
        .force("center", d3.forceCenter(width / 2, height / 2));

    const link = svg.append("g")
        .attr("stroke", "#999")
        .attr("stroke-opacity", 0.6)
        .selectAll("line")
        .data(data.links)
        .join("line")
        .attr("stroke-width", d => Math.sqrt(d.value));
//console.log(data.nodes);
    const node = svg.append("g")
        .attr("stroke", "#fff")
        .attr("stroke-width", 1.5)
        .selectAll("circle")
        .data(data.nodes)
        .join("circle")
        .attr("r", 10)
        .attr("fill", "steelblue")
        .call(drag(simulation));

    node.append("title")
        .text(d => `ID:${d.id}`);

    simulation.on("tick", () => {
        link
            .attr("x1", d => d.source.x)
            .attr("y1", d => d.source.y)
            .attr("x2", d => d.target.x)
            .attr("y2", d => d.target.y);

        node
            .attr("cx", d => d.x)
            .attr("cy", d => d.y);
    });

    // tooltip div
    const div = d3
        .select("body")
        .append("div")
        .attr("class", "tooltip")
        .style("opacity", 0);


    const adjacencySvg = d3.select(".adjacency")
        .attr("width", 930)
        .attr("height", 730)
        .style("background-color", "#e4e4e4")

    adjacencySvg.append("g")
        .attr("transform", "translate(200,50)")
        .attr("id", "adjacencyG")
        .selectAll("rect")
        .data(matrix)
        .enter()
        .append("rect")
        .attr("class", "grid")
        .attr("width", 20)
        .attr("height", 20)
        .attr("x", d => d.x * 20)
        .attr("y", d => d.y * 20)
        .style("fill-opacity", d => d.weight * .2)
        .on("mouseover", (event, d) => {
            div.transition()
                .duration(200)
                .style("opacity", .9);
            div.html(`x:${d.id.split("-")[0]}<br/> y:${d.id.split("-")[1]}`)
                .style("left", (event.pageX + 10) + "px")
                .style("top", (event.pageY - 20) + "px");
        })
        .on("mouseout", _ => {
            div.transition()
                .duration(500)
                .style("opacity", 0);
        })
        .on("mousemove", event => {
            div.style("left", (event.pageX + 10) + "px")
                .style("top", (event.pageY - 20) + "px")
        })
    console.log(data.links);
    adjacencySvg
        .append("g")
        .attr("transform", "translate(200,45)")
        .selectAll("text")
        .data(data.links)
        .enter()
        .append("text")
        .attr("x", (d, i) => i * 90 + 10)
        .text(d => d.name)
        .style("text-anchor", "middle")
        .style("font-size", "10px")

    adjacencySvg
        .append("g").attr("transform", "translate(185,50)")
        .selectAll("text")
        .data(data.links)
        .enter()
        .append("text")
        .attr("y", (d, i) => i * 30 + 15)
        .text(d => d.name)
        .style("text-anchor", "end")
        .style("font-size", "10px")
    
}


// loading data and then rendering chart
loadData()
.then(data => render(data))
.catch(err => console.log(err))